import openpyxl
import json
import re

from time import time
from datetime import datetime, date
from stringcolor import *

from sterra import _his
from sterra import _pri

class ort:
    def __init__(self, ld, pt, fm, tp=None, tu=None, dt='', cn=False):
        '''- ld: List of ?__a=1 dictionnaries
        - tp: Type of data contained
        - fm: Format in wich the file will be saved
        - tu: Instagram target's username
        - pt: Path where to write the file
        - dt: Data to write on the file infos case for excel
        - cn: Custom name'''

        if not fm in ['csv', 'excel', 'json']:
            raise ValueError('"fm" must be "csv", "excel" or "json".')

        xt = {
            'csv': '.csv',
            'excel': '.xlsx',
            'json': '.json'}

        self.ld = ld
        self.tp = tp
        self.fm = fm
        self.tu = tu
        if not cn:
            try:
                self.fn = pt+self.tu.replace('.', '_')+'_'+self.tp+'.'+date.today().strftime("%b-%d-%Y")+datetime.now().strftime(";%H-%M-%S")+xt[self.fm]
            except AttributeError:
                self.fn = pt+self.tp+'.'+date.today().strftime("%b-%d-%Y")+datetime.now().strftime(";%H-%M-%S")+xt[self.fm]
        else:
            self.fn = f'{pt}{cn}#{int(time())}{xt[self.fm]}'
        self.dt = dt if dt != '' else tp

    def EXcsv(self):
        cycle = 0
        for d in self.ld:
            if cycle == 0:
                open(self.fn, 'a').write(','.join(d.keys())+'\n')
            open(self.fn, 'a').write(','.join([str(d[key]).replace('\n', '<breakline>').replace(',', '<coma>') for key in d.keys()])+'\n')
            cycle += 1

    def MKexcel(self, cn):
        '''Create the excel file
        - cn: Cases name (type == list)'''
        def fileInfos(x):
            ExcelSheet.cell(row = 1, column = x, value = 'S T E R R A')
            ExcelSheet.cell(row = 2, column = x, value = '=HYPERLINK("https://github.com/novitae", "** Made By novitae **")').style='Hyperlink'
            ExcelSheet.cell(row = 4, column = x, value = 'Targeted account :')
            ExcelSheet.cell(row = 5, column = x, value = f'=HYPERLINK("https://www.instagram.com/{self.tu}/", "{self.tu}")').style='Hyperlink'
            ExcelSheet.cell(row = 6, column = x, value = 'Date :')
            ExcelSheet.cell(row = 7, column = x, value = date.today().strftime("%b-%d-%Y")+datetime.now().strftime(" %H:%M:%S"))
            ExcelSheet.cell(row = 9, column = x, value = 'Data :')
            ExcelSheet.cell(row = 10, column = x, value = self.dt)
        
        ExcelFile = openpyxl.Workbook()
        ExcelSheet = ExcelFile[ExcelFile.sheetnames[0]]
        for nx in range(len(cn)):
            ExcelSheet.cell(row=1, column=nx+1, value=cn[nx])
        fileInfos(len(cn)+1)
        ExcelFile.save(self.fn)

    def EXSexcel(self, d, r):
        '''Fills the excel file created before
        - d: Dictionnary
        - r: Row number'''
        r += 2
        ExcelFile = openpyxl.load_workbook(filename = self.fn)
        ExcelSheet = ExcelFile[ExcelFile.sheetnames[0]]
        c = 1
        for v in range(len(d.keys())):
            vl = list(d.values())
            if vl[v] != None:
                if re.findall(r'http[s]{0,1}:\/\/', str(vl[v])) != [] and not ' ' in str(vl[v]):
                    ExcelSheet.cell(row = r, column = c, value = f'=HYPERLINK("{vl[v]}", "{vl[v]}")').style='Hyperlink'
                else:
                    ExcelSheet.cell(row = r, column = c, value = type(vl[v])(vl[v]))
            c += 1
        ExcelFile.save(self.fn)

    def d(self):
        '''Destination'''
        open(self.fn, 'w').close()
        print(f'{_pri.i()}Writing file ...')
        if self.fm == 'csv':
            logo = _pri.CSV()
            ort.EXcsv(self)
        elif self.fm == 'excel':
            logo = _pri.Excel()
            ort.MKexcel(self, list(self.ld[0].keys()))
            for r in range(len(self.ld)):
                ort.EXSexcel(self, self.ld[r], r)
        elif self.fm == 'json':
            logo = _pri.Json()
            open(self.fn, 'w').write(json.dumps(self.ld, indent=4))

        print(f'{logo}File succesfully exported under '+bold(self.fn).underline())
        
        _his.tory(dd=self.fn)
        return self.fn


import os
import re
import random
import time
import json
import requests
import asyncio
import aiohttp

from sterra import _exp
from sterra import _log
from sterra import _pri
from sterra import _user

from stringcolor import *
from tqdm import tqdm

def keskonfé(rtn):
    while True:
        kskfé = str(input(_pri.Interrrogation()+f'Instagram blocked our requests during the process.\n    Do you prefer:\n    1 - Continue with what have been extracted,\n    2 - Export what have been exported as a part (link to part informations on wiki),\n    3 - Stop the program.\n    --> '))
        if kskfé == '1':
            return True

        elif kskfé == '2':
            open(part_path, 'w').write(json.dumps(rtn, indent=4))
            print(f'{_pri.Json()}Part merged to '+bold(part_path))
            exit()

        elif kskfé == '3':
            exit()

        else:
            print(_pri.Exception()+f' "{kskfé}" is not a valid answer, please try again.')

class traction:
    def __init__(self, ul, sd, ai=False, xp=False, dl=0) -> list:
        '''- ul: Usernames list
        - sd: Sessionid
        - ai: All infos ( True | False )
        - xp: Express mode ( True | False )
        - dl: Delay'''

        self.ua = _user.agent()
        self.ul = ul
        self.ck = {'sessionid': sd}
        self.ai = ai
        self.xp = xp
        self.dl = dl

    def keep_essential_data(ai, j):
        j = j['graphql']['user']
        accinfo = {
            'id': j['id'],
            'username': j['username'],
            'name': j['full_name'],
            'link': 'https://www.instagram.com/'+j['username']+'/',
            'biography': j['biography'],
            'is_private': j['is_private'],
            'followers': j['edge_followed_by']['count'],
            'following': j['edge_follow']['count'],
            'posts': j['edge_owner_to_timeline_media']['count'],
            'external_url': j['external_url'],
            'is_business_account': j['is_business_account'],
            'is_professional_account': j['is_professional_account'],
            'is_verified': j['is_verified']}
        if ai:
            accinfo['business_address_json'] = j['business_address_json']
            accinfo['business_category_name'] = j['business_category_name']
            accinfo['business_contact_method'] = j['business_contact_method']
            accinfo['business_email'] = j['business_email']
            accinfo['business_phone_number'] = j['business_phone_number']
            accinfo['connected_fb_page'] = j['connected_fb_page']
            accinfo['mutual_followed_by_count'] = j['edge_mutual_followed_by']['count']
            accinfo['has_ar_effects'] = j['has_ar_effects']
            accinfo['has_channel'] = j['has_channel']
            accinfo['has_clips'] = j['has_clips']
            accinfo['has_guides'] = j['has_guides']
            accinfo['hide_like_and_view_counts'] = j['hide_like_and_view_counts']
            accinfo['is_joined_recently'] = j['is_joined_recently']

        return accinfo

    def get_a__1_normal(self, rtn):
        progressbar = tqdm(self.ul, desc=_pri.Requests()+bold('Details extraction '), colour='Black')
        for usrnm in progressbar:
            __a1 = requests.get(f'https://www.instagram.com/{usrnm}/channel/?__a=1', headers = {'User-Agent': self.ua, 'Referer': f'https://www.instagram.com/{usrnm}'}, cookies=self.ck, allow_redirects=False)
            time.sleep(self.dl)
            if __a1.status_code == 200 and __a1.text != {}:
                rtn.append(__a1.json())
            else:
                progressbar.close()
                if keskonfé(rtn):
                    return rtn
        return rtn

    async def get_a__1_express(self, rtn):
        async def fetch(session, url):
            async with session.get(url, headers = {'User-Agent': self.ua, 'Referer': f'https://www.instagram.com/'}, cookies=self.ck, allow_redirects=False) as __a1:
                if __a1.status == 200 and await __a1.text() != {}:
                    progressbar.update(1)
                    rtn.append(await __a1.json())
                else:
                    progressbar.close()
                    if keskonfé(rtn):
                        return rtn
            return rtn

        progressbar = tqdm(self.ul, desc=_pri.Requests()+bold('Details extraction '), colour='Black')
        async with aiohttp.ClientSession() as session:
            requests_corroutines = []
            for usrnm in self.ul:
                requests_corroutines.append(fetch(session, f'https://www.instagram.com/{usrnm}/channel/?__a=1'))
            futures = asyncio.gather(*requests_corroutines)
            await futures
            await session.close()
        
        progressbar.reset()
        progressbar.update(len(self.ul))
        progressbar.close()

        return rtn

    def strt(self, rtn):
        if not self.xp or len(self.ul) > 110:
            if len(self.ul) > 10:
                print(cs('(', 'Plum3')+bold('☇').cs('Plum3')+cs(')', 'Plum3')+' Details extraction '+cs('can take a while', 'White').bold().underline()+'.\n    You are free to '+cs.bold('do something else').underline()+' while the program does its job.')
            jsonlist = traction.get_a__1_normal(self, rtn)
        elif self.xp:
            loop = asyncio.get_event_loop()
            jsonlist = loop.run_until_complete(traction.get_a__1_express(self, rtn))

        return [traction.keep_essential_data(self.ai, j) for j in jsonlist]

followersList = []
followingList = []
probabilities = {}
target_username = []

def id_to_username(i, u, s):
    '''Returns the username from the ID of an account.'''
    j = requests.get(f'https://i.instagram.com/api/v1/users/{i}/info/', headers={'User-Agent': u}, cookies = {'sessionid': s}, allow_redirects=False)
    if j.status_code == 200:
        return j.json()['user']['username']
    elif j.status_code == 404: _pri.outExcept(e=f'TargetException: The target account associated to "{i}" ID does not exist.')
    else: _pri.outExcept(e=f'ResponseException: The API responded {j.status_code}, you might have been blocked.')

def usernameCheck(u, h, s):
    '''Checks the account existence, and returns its id, followers and following count if it exists, else raises a SterraException'''
    __a1 = requests.get(f'https://www.instagram.com/{u}/channel/?__a=1', cookies = {'sessionid': s}, headers={'User-Agent': h}, allow_redirects=False)
    try:
        if __a1.status_code == 302:
            _pri.outExcept(e=f'ResponseException:\n    Check if your filled well your SessionID. If you did, or logged with an Username and Password.')
        elif __a1.status_code == 404 or __a1.json() == {}:
            _pri.outExcept(e=f'TargetException: The target account "{u}" does not exist.')
        elif __a1.status_code == 200 and __a1.json() != {}:
            j = __a1.json()['graphql']['user']
            return {'id': j['id'], 'followers': j['edge_followed_by']['count'], 'following': j['edge_follow']['count']}
        else:
            _pri.outExcept(e=f'ResponseException: Response {__a1.status_code}, you might have been blocked by instagram.\n    You have no other options than waiting or changing of account.')
    except json.decoder.JSONDecodeError:
        _pri.outExcept(e=f'ResponseException: API rate limit reached, wait an hour or try to use another account.')

class get_follow_list:
    '''Retrieve the followers or following list of an instagram account.'''
    def __init__(self, targid, cred=None, target='following',) -> list:
        '''- targid: The instagram username or id of the account to aim for.
        - cred: SessionID in str or list of credentials like so ["username", "password"] of  the instagram account you want to use.
        - target: The list you want to extract, "following", "followers" or "both". It will be "following" by default.'''

        global sssid
        sssid = _log.in_sssid(cred)
        self.sssid = sssid
        self.trget = target
        self.usrgt = _user.agent()
        reco, tpe = targid # RÉCUPÈRE L'USERNAME SI LE COMPTE VISÉ EST UN ID
        self.usrnm = reco if tpe == 'username' else id_to_username(reco, self.usrgt, self.sssid)

        infos = usernameCheck(self.usrnm, self.usrgt, self.sssid)
        self.trgid = infos['id']
        self.infos = {'followersCount': infos['followers'], 'followingCount': infos['following']}
        print(_pri.x()+cs('Target account', 'Red2').underline()+cs.bold(' >>> ')+cs(self.usrnm, 'White', 'Red2').bold()+', '+cs.bold(str(self.infos['followersCount']))+' followers, '+cs.bold(str(self.infos['followingCount']))+' following.')
        
        if self.trget in ['following', 'both', 'mutuals'] and self.infos['followersCount'] == 0:
            _pri.outExcept(e=f'TargetException: Argument "{self.trget}" needs a list longer than 0 following.')
        if self.trget in ['followers', 'both', 'mutuals'] and self.infos['followingCount'] == 0:
            _pri.outExcept(e=f'TargetException: Argument "{self.trget}" needs a list longer than 0 following.')
        target_username.append(self.usrnm)

    def ig_request(hash_id, variables, resolver, cookies, count, sleep_error = 10, tentatives = 3):
        '''This part of code is pasted from https://github.com/tuxity/insta-unfollower/blob/master/insta-unfollower.py, made to retrieve list of following or follower by 25 unity.'''
        has_next_page = True
        tentatives_actuelles = 0
        
        params = {"query_hash": hash_id, "variables": json.dumps(variables)}
        
        bfadd = 0
        fllwExtrct = tqdm(total=int(count), desc=(_pri.Instagram()+(cs.bold('Followers') if hash_id == 'c76146de99bb02f6415203be841dd25a' else cs.bold('Following'))+' extraction '), colour='Black')
        
        while has_next_page and tentatives_actuelles < tentatives:
            resp = requests.get("https://www.instagram.com/graphql/query/", params = params, cookies = cookies)
            respcookies = resp.cookies.get_dict()
            if respcookies == {}:
                fllwExtrct.close()
                _pri.outExcept(e='TargetException: Be sure that you are following the account you are aiming for if it is in private.')
            
            if resp.status_code == 200:
                tentatives_actuelles = 0

                bfadd = len(followersList if hash_id == 'c76146de99bb02f6415203be841dd25a' else followingList)
                has_next_page = resolver(variables, resp.json())
                fllwExtrct.update(len(followersList if hash_id == 'c76146de99bb02f6415203be841dd25a' else followingList) - bfadd)
                if has_next_page:
                    params["variables"] = json.dumps(variables)

            else:
                tentatives_actuelles += 1
                print(_pri.Exception()+f" An error happened, retry {tentatives_actuelles}/{tentatives} ({sleep_error} seconds between each retry).")
                time.sleep(sleep_error)

        fllwExtrct.reset()
        fllwExtrct.update(int(count))
        fllwExtrct.close()
        return tentatives_actuelles < tentatives

    def resolver_followers(variables, data_resp):
        '''This part of code is pasted from https://github.com/tuxity/insta-unfollower/blob/master/insta-unfollower.py, made to ask ig_request for 25 followers at time'''
        data_resp = data_resp["data"]["user"]["edge_followed_by"]
        
        for node in data_resp["edges"]:
            followersList.append(node["node"]["username"])
                
        if data_resp["page_info"]["has_next_page"]:
            variables["after"] = data_resp["page_info"]["end_cursor"]
            return True
        else:
            return False

    def resolver_following(variables, data_resp):
        '''This part of code is pasted from https://github.com/tuxity/insta-unfollower/blob/master/insta-unfollower.py, made to ask ig_request for 25 following at time'''
        data_resp = data_resp["data"]["user"]["edge_follow"]
        
        for item in data_resp["edges"]:
            followingList.append(item["node"]["username"])
                
        if data_resp["page_info"]["has_next_page"]:
            variables["after"] = data_resp["page_info"]["end_cursor"]
            return True
        else:
            return False

    def generate(self):
        '''Returns the list of every follower or/and following (depends on the arg "target"=???) under a dictionnary containing both'''
        cookies = {'sessionid': self.sssid}
        if self.trget == 'followers':
            get_follow_list.ig_request(hash_id="c76146de99bb02f6415203be841dd25a", variables={"id": self.trgid, "first": 50}, resolver=get_follow_list.resolver_followers, cookies = cookies, count=int(self.infos['followersCount']))
        elif self.trget == 'following':
            get_follow_list.ig_request(hash_id="d04b0a864b4b54837c0d870b0e77e076", variables={"id": self.trgid, "first": 50}, resolver=get_follow_list.resolver_following, cookies = cookies, count=int(self.infos['followingCount']))
        elif self.trget == 'both' or self.trget == 'mutuals':
            get_follow_list.ig_request(hash_id="c76146de99bb02f6415203be841dd25a", variables={"id": self.trgid, "first": 50}, resolver=get_follow_list.resolver_followers, cookies = cookies, count=int(self.infos['followersCount']))
            get_follow_list.ig_request(hash_id="d04b0a864b4b54837c0d870b0e77e076", variables={"id": self.trgid, "first": 50}, resolver=get_follow_list.resolver_following, cookies = cookies, count=int(self.infos['followingCount']))

        if followersList == [] and self.trget == 'followers' and int(self.infos['followersCount']) > 0 or followersList == [] and self.trget == 'both' and int(self.infos['followersCount']) > 0:
            _pri.outExcept(e=f'TargetException: Sterra didn\'t retrieve any username from {self.usrnm}\'s followers. Verify that you have access to this account if it is private.')
        if followingList == [] and self.trget == 'following' and int(self.infos['followingCount']) > 0 or followingList == [] and self.trget == 'both' and int(self.infos['followingCount']) > 0:
            _pri.outExcept(e=f'TargetException: Sterra didn\'t retrieve any username from {self.usrnm}\'s following. Verify that you have access to this account if it is private.')

        return (followersList, followingList)

def makemutuals(lufollowing, lufollowers):
    mutualList = []
    mlngliste = lufollowing if len(lufollowing) <= len(lufollowers) else lufollowers
    plngliste = lufollowers if len(lufollowers) >= len(lufollowing) else lufollowing
    for flw in mlngliste: # LISTE DE MUTUELS
        if flw in plngliste:
            mutualList.append(flw)
    return mutualList

def mk_converted(rm, lufollowers, lufollowing):
    converted = []
    if rm is not None:
        try:
            filename = rm.split('/')[-1]
            if not re.match(r'(followers|following|both)#[0-9]{10}\.json', filename):
                _pri.outExcept(e='Not compatible part file')
            infos = filename.split('#')
            '''target username # followers or following # timestamp . json'''
            
            from sterra import _con
            converted = _con.verter(rm)
            lusername = [U['graphql']['user']['username'] for U in converted]
            if infos[0] == 'followers':
                lufollowers = [F for F in lufollowers if not F in lusername]
            elif infos[0] == 'following':
                lufollowing = [F for F in lufollowing if not F in lusername]
            else: # both, removed but we never know
                lufollowers = [F for F in lufollowers if not F in lusername]
                lufollowing = [F for F in lufollowing if not F in lusername]

        except FileNotFoundError:
            _pri.outExcept(e=f'File supposed to be at {rm} has not been found.')

    return {
        'lufollowers': lufollowers,
        'lufollowing': lufollowing,
        'converted': converted
        }

def tagram(tu, cd, tl, pt, ex=False, ai=False, fm='excel', dl=0, rm=None):
    '''- tu: Target's username like this : ("your target username", "username"), or target's ID like this : ("your target id", "id")
    - cd: Credential ; sessionid (in string) or credentials like this : ["username", "password"] (in list)
    - tl: Target list : following - followers - both - mutuals
    - pt: File path
    - ex: Express mode ( Yes = True | No = False)
    - ai: All infos ( Yes = True | No = False)
    - fm: Format of the file to export : excel - csv - json'''
    ll = get_follow_list(targid=tu, cred=cd, target=tl).generate()
    lufollowers, lufollowing = ll

    global part_path
    part_path = rm if rm is not None else os.path.dirname(__file__)+f'/export/parts/{tl}#{int(time.time())}.json'
    
    if rm is not None and tl in ['both', 'mutuals']:
        _pri.outExcept(e='Part function must be targetting followers or following, not both or mutuals')
    mk = mk_converted(rm, lufollowers, lufollowing)

    if tl in ['followers', 'following', 'both']:
        if tl in ['followers', 'both']:
            followers = traction(ul=mk['lufollowers'], sd=sssid, ai=ai, xp=ex, dl=dl).strt([] if tl in ['following', 'both'] else mk['converted'])
            _exp.ort(ld=followers, tp='followers', fm=fm, tu=target_username[0], pt=pt).d()

        if tl in ['following', 'both']:
            following = traction(ul=mk['lufollowing'], sd=sssid, ai=ai, xp=ex, dl=dl).strt([] if tl in ['followers', 'both'] else mk['converted'])
            _exp.ort(ld=following, tp='following', fm=fm, tu=target_username[0], pt=pt).d()

    elif tl == 'mutuals':
        mutualList = makemutuals(lufollowing, lufollowers)
        if mutualList == []:
            _pri.outExcept(e=f'DataException: Sterra did\'nt find any mutual follows accounts on {tu} account.')
        else:
            mutuals = traction(ul=mutualList, sd=sssid, ai=ex, xp=ex, dl=dl).strt()
            _exp.ort(ld=mutuals, tp='mutuals', fm=fm, tu=target_username[0], path=pt).d()


import re

class babilities:
    def __init__(self, ld, pa, io = 10000, dd = False, pt = 0, sz=None) -> dict:
        '''- ld: List of dics
        - pa: Probability asked -> personnal - interests
        - io: Ignore over -> Amount of followers over wich the account is considerated as non-personnal
        - dd: Return by descending order ? True - False
        - pt: Percentage
        - sz: Size of the list returned'''

        if type(ld) != list: raise TypeError('InputTypeException: "lofdic" must be a list.')
        if ld == []: raise ValueError('InputTypeException: "lofdic" must be a not empty list.')
        if type(ld[0]) != dict: raise TypeError('elements of "lofdic" must be dictionnaries.')
        if type(io) != int: raise ValueError('"ignore_over" must be an integer.')
        
        self.ld = ld
        self.pa = pa
        self.io = io
        self.pt = pt
        self.dd = False if dd is True else True
        self.sz = sz if sz is not None else len(ld)

    def sort(self, l):
        srt = sorted(l, key=lambda item: item['%'], reverse=self.dd)
        srt = [x for x in srt if int(x['%']) >= self.pt]
        return srt[:self.sz]

    def coef(p, c, v): #c = coefficient
        for _ in range(c):
            p.append(v)
        return p

    def personnal(self):
        o = []
        for D in self.ld:
            d = {}
            for key in list(D.keys()):
                try:
                    d[key] = int(D[key])
                except (ValueError, TypeError):
                    d[key] = D[key]

            '''Objective probababilities'''
            p = []
            repv = r'([pP]{1,4}[rR]{1,4}([iI]{0,4}[vV]{1,4}[aAéÉeE]{1,10}[tT]{0,4}[eE]{0,4}|[vV]{1,4}[tT]{1,4}[eE]{0,9})|[pP][vV]{1,4})' # PRIVATE / PRIVÉ
            if d['biography'] != None:
                if re.findall(repv, str(d['biography'])) != []:
                    p = babilities.coef(p, c=2, v=100)

            if d['name'] != None:
                if re.findall(repv, str(d['name'])) != []:
                    p = babilities.coef(p, c=2, v=100)

            if re.findall(repv, str(d['username'])) != []:
                p = babilities.coef(p, c=2, v=100)
            p = babilities.coef(p, c=4, v=(100 if d['is_private'] else 0))
            p = babilities.coef(p, c=5, v=(100 - (int(d['followers'] if d['followers'] < self.io else self.io) * 100) / self.io))
            p = babilities.coef(p, c=1, v=(100 - (int(d['following'] if d['following'] < 1500 else 1500) * 100) / 1500))
            p = babilities.coef(p, c=1, v=(0 if d['external_url'] != None else 100))

            '''Subjective probababilities
            Data is added in function of the first estimations'''
            sub = 0
            for i in p:
                sub += i
            SUB = int(sub / len(p))

            p = babilities.coef(p, c=1, v=100) if not d['is_professional_account'] else babilities.coef(p, c=1, v=(100 if SUB > 75 else 0))
            p = babilities.coef(p, c=1, v=100) if not d['is_business_account'] else babilities.coef(p, c=1, v=(100 if SUB > 75 else 0))
            p = babilities.coef(p, c=2, v=100) if not d['is_verified'] else babilities.coef(p, c=3, v=(100 if SUB > 60 else 0))
            p = babilities.coef(p, c=2, v=(100 if d['posts'] > 50 and SUB > 92 else 0))

            t = 0
            for i in p: t += i
            T = int(t / len(p))
            T = T if T != 100 else 99

            oa = {'%': int(str(T if T > 9 else '0'+str(T)))}
            kys = list(d.keys())
            for k in range(len(kys)):
                oa[kys[k]] = list(d.values())[k]
            o.append(oa)
        return o

    def interests(self):
        pass

    def d(self):
        '''Destination'''
        if self.pa == 'personnal':
            return babilities.sort(self, babilities.personnal(self))
        elif self.pa == 'interests':
            return babilities.sort(self, babilities.interests(self))


from random import choice

List = [
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone12,8; iOS 14_8; en_CL; en-CL; scale=2.00; 750x1334; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone11,8; iOS 15_0_1; es_CL; es-CL; scale=2.00; 828x1792; 325544617)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,1; iOS 14_7_1; en_SA@calendar=gregorian; en-SA; scale=2.88; 1080x2338; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,1; iOS 14_7_1; en_SA@calendar=gregorian; en-SA; scale=2.88; 1080x2338; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone14,2; iOS 15_1; da_DK; da-DK; scale=3.00; 1170x2532; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8; pt_BR; pt-BR; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,3; iOS 14_7_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 329643252) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,4; iOS 15_1; it_IT; it; scale=3.00; 1284x2778; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,1; iOS 15_0_2; ru_RU; ru; scale=2.88; 1080x2338; 330663239)",
    "Mozilla/5.0 (iPad; CPU OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPad8,9; iPadOS 15_1; en_IT; en-GB; scale=2.00; 750x1334; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 13_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone12,3; iOS 13_5_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 322184766) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 13_5_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone12,3; iOS 13_5_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 322184766)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8_1; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8_1; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,2; iOS 15_0; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone9,1; iOS 14_8_1; it_IT; it-IT; scale=2.00; 750x1334; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_7_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone9,3; iOS 13_6; ru_RU; ru-RU; scale=2.00; 750x1334; 322184766) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 15_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 15_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone11,8; iOS 15_0_2; it_IT; it-IT; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone9,1; iOS 14_7_1; ru_RU; ru-RU; scale=2.00; 750x1334; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone9,4; iOS 15_1; ru_RU; ru-RU; scale=2.61; 1080x1920; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone9,4; iOS 15_1; ru_RU; ru-RU; scale=2.61; 1080x1920; 327311214) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,2; iOS 15_0; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,2; iOS 15_0; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone10,4; iOS 15_0_2; ru_RU; ru-RU; scale=2.00; 750x1334; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone10,4; iOS 15_0_2; ru_RU; ru-RU; scale=2.00; 750x1334; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone9,3; iOS 15_0_1; ru_RU; ru-RU; scale=2.00; 750x1334; 325544617) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone9,3; iOS 15_0_1; ru_RU; ru-RU; scale=2.00; 750x1334; 325544617) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 15_0_2; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone9,3; iOS 15_0_1; ru_RU; ru-RU; scale=2.00; 750x1334; 325544617)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 15_0_2; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 13_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 208.0.0.26.131 (iPhone9,3; iOS 13_6; ru_RU; ru-RU; scale=2.00; 750x1334; 322184766)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 210.0.0.16.67 (iPhone8,1; iOS 15_1; ru_RU; ru-RU; scale=2.00; 750x1334; 325544617)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,3; iOS 15_1; en_IT; en-IT; scale=3.00; 1125x2436; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone8,1; iOS 15_0; ru_RU; ru-RU; scale=2.00; 750x1334; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8; ru_AT; ru-AT; scale=3.00; 1170x2532; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_8; ru_AT; ru-AT; scale=3.00; 1170x2532; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone11,8; iOS 15_1; it_GB; it-GB; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 123.0.0.24.115 (iPhone10,4; iOS 14_8; en_IT; en-GB; scale=2.00; 750x1334; 188362626)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone10,5; iOS 15_0_2; it_IT; it-IT; scale=2.61; 1080x1920; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone11,8; iOS 14_8_1; ru_RU; ru-RU; scale=2.00; 828x1792; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,1; iOS 14_8_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone9,3; iOS 14_7_1; it_IT; it-IT; scale=2.00; 750x1334; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone14,3; iOS 15_0_2; it_IT; it-IT; scale=3.00; 1284x2778; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 15_0_2; it_IT; it-IT; scale=3.00; 1170x2532; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,1; iOS 14_7_1; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 207.0.0.28.118 (iPhone8,1; iOS 14_4_1; ru_RU; ru-RU; scale=2.00; 750x1334; 320361397)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,1; iOS 14_7_1; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_4_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone13,4; iOS 14_4_2; ru_RU; ru-RU; scale=3.00; 1284x2778; 327311214) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,3; iOS 14_7_1; ru_RU; ru-RU; scale=3.00; 1170x2532; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,3; iOS 15_0_2; it_IT; it-IT; scale=3.00; 1170x2532; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone9,3; iOS 14_8_1; ru_RU; ru-RU; scale=2.00; 750x1334; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone11,8; iOS 15_0_2; it_IT; it-IT; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 206.0.0.30.118 (iPhone13,2; iOS 14_6; it_IT; it-IT; scale=3.00; 1170x2532; 318760365)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone12,5; iOS 15_0_2; ru_RU; ru-RU; scale=3.00; 1242x2688; 330663239)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone11,2; iOS 15_1; it_IT; it-IT; scale=3.00; 1125x2436; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone8,1; iOS 14_2; it_IT; it-IT; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,2; iOS 14_8; en_US; en-US; scale=3.66; 1170x2533; 329643252) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,2; iOS 14_8; en_US; en-US; scale=3.66; 1170x2533; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone12,1; iOS 14_8; pt_BR; pt-BR; scale=2.00; 828x1792; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 209.1.0.25.113 (iPhone9,4; iOS 14_8_1; ru_RU; ru-RU; scale=2.61; 1080x1920; 324511477)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone9,3; iOS 14_8; ru_RU; ru-RU; scale=2.00; 750x1334; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 15_0_2; it_IT; it-IT; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone11,8; iOS 14_7_1; it_IT; it-IT; scale=2.00; 828x1792; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,2; iOS 15_0_2; ru_RU; ru-RU; scale=2.61; 1080x1920; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone12,1; iOS 14_7_1; it_IT; it-IT; scale=2.00; 828x1792; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_6 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone10,5; iOS 14_6; it_IT; it-IT; scale=2.61; 1080x1920; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone13,2; iOS 15_0_2; it_IT; it-IT; scale=3.00; 1170x2532; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_4 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone8,1; iOS 14_4; it_IT; it-IT; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone11,8; iOS 14_8_1; it_IT; it-IT; scale=2.00; 828x1792; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,1; iOS 15_0_2; en_IT; en-IT; scale=2.21; 828x1792; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 213.0.0.19.117 (iPhone10,5; iOS 14_8_1; pt_AL; pt-AL; scale=2.88; 1080x1920; 330663239) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 15_1; it_IT; it-IT; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone10,5; iOS 14_8_1; ru_RU; ru-RU; scale=2.61; 1080x1920; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone10,5; iOS 14_8_1; ru_RU; ru-RU; scale=2.61; 1080x1920; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 14_3; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 15_0_1; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone10,6; iOS 15_1; it_IT; it-IT; scale=3.00; 1125x2436; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone9,3; iOS 14_7_1; ru_KZ; ru-KZ; scale=2.00; 750x1334; 327311214)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,1; iOS 15_2; ru_RU; ru-RU; scale=2.00; 828x1792; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone11,2; iOS 14_7_1; ru_RU; ru-RU; scale=3.00; 1125x2436; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,2; iOS 15_0_2; ru_RU; ru-RU; scale=2.61; 1080x1920; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone11,8; iOS 15_1; es_AR; es-AR; scale=2.21; 828x1792; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,1; iOS 14_8_1; en_GB; en-GB; scale=2.00; 828x1792; 329643252) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 14_7_1; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 14_7_1; ru_RU; ru-RU; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone13,2; iOS 14_7_1; it_IT; it; scale=3.00; 1170x2532; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,4; iOS 15_0_2; ru_RU; ru-RU; scale=2.88; 1080x1920; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,4; iOS 15_0_2; ru_RU; ru-RU; scale=2.88; 1080x1920; 329643252)",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone12,1; iOS 14_8_1; en_GB; en-GB; scale=2.00; 828x1792; 329643252) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 15_0_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 211.0.0.21.118 (iPhone10,4; iOS 15_0_2; it_IT; it-IT; scale=2.00; 750x1334; 327311214) NW/3",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_4_2 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone9,3; iOS 14_4_2; it_IT; it-IT; scale=2.00; 750x1334; 329643252)",
    "Mozilla/5.0 (iPad; CPU OS 14_8_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPad7,2; iOS 14_8_1; it_IT; it-IT; scale=2.00; 1334x750; 329643252) NW/1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 14_7_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/15E148 Instagram 212.1.0.25.118 (iPhone10,4; iOS 14_7_1; it_IT; it-IT; scale=2.00; 750x1334; 329643252)"
]

def agent():
    return choice(List)


import os
import json
from stringcolor import *
from sterra import _pri

def recreate(PATH):
    with open(PATH, 'w') as a:
        a.write(json.dumps([], indent=4))

DIR = os.path.dirname(__file__)+'/'
PATH = DIR+'EXPORTS-HISTORY.json'
if not os.path.exists(PATH):
    recreate(PATH)

def tory(dd=None, ll=False, nm=None, cl=False):
    '''- dd: Add a str to the historic (experimental),
    - ll: True to print all the historic
    - nm: Prints all path containing the str of nm
    - cl: True to clear the historic'''
    if dd is not None:
        j_add = json.loads(open(PATH, 'r').read())
        j_add.append(dd)
        with open(PATH, 'w') as b:
            b.write(json.dumps(j_add, indent=4))
    
    elif cl:
        validation = input(_pri.Interrrogation()+'Are you sure you want to erase your export historic ? '+bold('There is no coming back after erasing it')+' !\n    ( '+bold('Y')+' | '+bold('N')+' )  --->  ')
        if validation.lower() == 'y':
            recreate(PATH)
            print(_pri.i()+f'Historic erased.')
        else:
            print(_pri.i()+f'Historic not erased.')
    else:
        def output(e):
            print(cs(' >  ', 'DodgerBlue2').bold()+underline(e))
        
        j_ra = json.loads(open(PATH, 'r').read())
        if j_ra == []:
            print(_pri.Exception()+'Empty historic')
        if ll:
            print(_pri.i()+'Historic of exportation paths:')
            for e in j_ra:
                output(e)
        
        elif nm is not None:
            print(_pri.i()+f'Searching for {nm} in exports historic:')
            notfound = True
            for e in j_ra:
                if nm in e:
                    output(e)
                    if notfound:
                        notfound = False
            if notfound:
                print(_pri.i()+f'Nothing including {nm} found in the historic')


from stringcolor import *

def Python():
    return cs('|', 'DeepSkyBlue5')+bold('§').cs('LightGoldenrod2', 'DeepSkyBlue5')+cs('|', 'LightGoldenrod2')+' '

def Excel():
    return cs('|', 'Green4')+cs('X', 'White', 'Green4')+cs('|', 'Green4')+' '

def Json():
    return cs('{', 'Purple5').bold()+cs('j', 'DeepSkyBlue4').bold()+cs('}', 'Purple5').bold()+' '

def Instagram():
    return cs('|', 'LightSalmon2')+cs('o', 'White', 'DeepPink4')+cs('|', 'MediumOrchid')+' '

def Requests():
    return cs('|', 'SeaGreen2')+bold('$').cs('White', 'DarkGrey')+cs('|', 'LightSalmon')+' '

def CSV():
    return cs('|', 'Orange')+bold('c').cs('White', 'Orange')+cs('|', 'Orange')+' '

def i():
    return cs('|', 'DodgerBlue2')+bold('i').cs('White', 'DodgerBlue2')+cs('|', 'DodgerBlue2')+' '

def Exception():
    return cs('|', 'Red2')+bold('!').cs('White', 'Red2')+cs('|', 'Red2')+' '

def Interrrogation():
    return cs('|', 'Violet')+bold('?').cs('White', 'Violet')+cs('|', 'Violet')+' '

def Plus():
    return cs('|', 'DarkSeaGreen9')+bold('+').cs('DarkSeaGreen9', 'White')+cs('|', 'DarkSeaGreen9')+' '

def x():
    return cs('|', 'Red2')+bold('x').cs('Red2')+cs('|', 'Red2')+' '

def compare():
    return cs('|', 'DeepSkyBlue7')+bold('≈').cs('White', 'DeepSkyBlue7')+cs('|', 'DeepSkyBlue7')+' '

def outExcept(e):
    print(Exception()+e)
    exit()

def logo(L):
    mainL = '~'+cs(' Sterra 2.1 ', 'MediumPurple6')+'~'+cs(' made by ', 'MediumPurple6')+bold(cs('aet', 'MediumPurple6'))+cs(' € https://github.com/novitae/sterra ', 'MediumPurple6')+'~'+cs(' GPL-3 ', 'MediumPurple6')+'~'
    logos = {
        'export': '~'+cs(' Export Submodule ', 'MediumPurple6')+'~ '+''.join([str(bold(cs(l, ['DodgerBlue2', 'SlateBlue2', 'SlateBlue', 'Pink4', 'LightPink3', 'SandyBrown', 'Gold', 'Yellow2', 'Yellow'][i], 'Black'))) for i, l in enumerate('instagram')])+' --> ( '+' | '.join([str(bold(cs(l, 'White', ['Green4', 'DarkOrange', 'Purple5'][i]))) for i, l in enumerate(['excel', 'csv', 'json'])])+' ) ~'+cs(' @#S%?*+;:- ', 'MediumPurple6')+'~',
        'compare': '~'+cs(' Compare Submodule ', 'MediumPurple6')+'~ '+f'{Excel()}{CSV()}{Json()}== != <= {Json()}{CSV()}{Excel()}'+'~'+cs(' @#$S%?*+;:-,. ', 'MediumPurple6')+'~',
        'analyse': '~'+cs(' Analyse Submodule ', 'MediumPurple6')+'~ '+f'{Excel()}--> :'+underline(cs('nt(x[\'%\'])>self.p];return s[:e', 'White', 'DarkGreen'))+': ~'+cs(' #%*;- ', 'MediumPurple6')+'~',
        'history': '~'+cs(' History Submodule ', 'MediumPurple6')+'~ '+f'36:28->"PATH", 16:34:57->"PATH", 03:12:4'+' ~'+cs(' #%*;- ', 'MediumPurple6')+'~',
        'convert': '~'+cs(' Convert Submodule ', 'MediumPurple6')+'~ '+f' {Excel()}-->{CSV()}-->{Json()}-->{Excel()}-->{CSV()}-->{Json()} '+'~'+cs(' #%*;- ', 'MediumPurple6')+'~',
    }
    return mainL+'\n'+logos[L]


import requests
import time
import re
import random

from datetime import datetime
from sterra import _pri
from sterra import _user
'''Probablement utilisé de manière plus fréquente lors de prochaines maj en imaginant une liste d'identifiants'''
def in_(u, p):
    '''Returns the sessionid
    - u: username
    - p: password'''
    
    session = requests.Session()
    session.headers.update({
        'Accept-Encoding': 'gzip, deflate',
        'Accept-Language': 'en-US,en;q=0.8',
        'Connection': 'keep-alive',
        'Content-Length': '0',
        'Host': 'www.instagram.com',
        'Origin': 'https://www.instagram.com',
        'Referer': 'https://www.instagram.com/',
        'User-Agent': _user.agent(),
        'X-Instagram-AJAX': '7a3a3e64fa87',
        'X-Requested-With': 'XMLHttpRequest'})

    reponse = session.get('https://www.instagram.com')
    csrf = re.findall(r"csrf_token\":\"(.*?)\"", reponse.text)[0]
    if csrf: session.headers.update({'x-csrftoken': csrf})
    else: _pri.outExcept(e='LoginException: No csrf token found in cookies. Wait 1 hour and retry, or change of account.')
    time.sleep(random.randint(1, 3))

    post_data = {'username': u, 'enc_password': '#PWD_INSTAGRAM_BROWSER:0:{}:{}'.format(int(datetime.now().timestamp()), p)}

    response = session.post('https://www.instagram.com/accounts/login/ajax/', data=post_data, allow_redirects=True)
    response_data = response.json()

    if 'two_factor_required' in response_data:
        _pri.outExcept(e='LoginException: Please disable 2-factor authentication to login.')
    if 'message' in response_data and response_data['message'] == 'checkpoint_required':
        _pri.outExcept(e='LoginException: Please check Instagram app for a security confirmation that it is you trying to login.')
    if not response_data['authenticated']:
        _pri.outExcept(e='LoginException: Login failed. Be sure your of your credentials.')

    return response.cookies['sessionid']

def in_sssid(c):
    if type(c) == list: # PAR IDENTIFIANTS
        sssid = in_(c[0], c[1])
    else: # PAR SESSIONID
        if re.match(r'[0-9]{1,32}%[0-9a-zA-Z]{16}%[0-9A-Z]{3,4}', c): sssid = c
        else: _pri.outExcept(e='LoginException: The provided sessionId doesn\'t match normal sessionId format. Verify that you wrote it correctly.\n    If the error is not going away, report it in issue, and write in it this regex "r\'[0-9]{1,32}%[0-9a-zA-Z]{16}%[0-9A-Z]{3,4}\'" with your report.')
    
    return sssid


import openpyxl
import re
import json
from sterra._pri import outExcept

def getnameinfos(pt):
    # NOM DU FICHIER
    name = pt.split('/')[-1]
    if not '#' in pt:
        # TYPE DE LISTE
        tp = re.findall(r'[a-z]{1,16}\.[A-Za-z]{3}-[0-9]{2}-[0-9]{4};[0-9]{2}-[0-9]{2}', name)[0].split('.')[0]
        # FORMAT
        fm = re.findall(r'(xlsx|csv|json)', name.split('.')[-1])[0]
        # TARGET USERNAME
        tu = '_'.join(re.findall(r'[a-z0-9_]{1,30}_'+tp, pt)[0].split('_')[:-1])
        # PATH
        pt = '/'.join(pt.split('/')[:-1])+'/'

    else:
        splt = name.split('#')
        # TYPE DE LISTE
        tp = splt[0]
        # FORMAT
        fm = re.findall(r'(xlsx|csv|json)', name.split('.')[-1])[0]
        tu = None
        # PATH
        pt = '/'.join(pt.split('/')[:-1])+'/'
    
    return {
        'username': tu,
        'type': tp,
        'path': pt,
        'name': name,
        'format': fm if fm != 'xlsx' else 'excel'
        }

def checkpathformat(pt):
    if re.match(r'[a-z_]{3,48}\.[A-Za-z]{3}-[0-9]{2}-[0-9]{4};[0-9]{2}-[0-9]{2}\.[a-z]{3,4}', getnameinfos(pt)['name']):
        return True
    return False

def excel(tr):
    ExcelFile = openpyxl.load_workbook(tr)
    ExcelSheet = ExcelFile[ExcelFile.sheetnames[0]]
    
    Colonnes = ExcelSheet.max_column
    Rangées = ExcelSheet.max_row
    Retour = []
    
    for r in range(2, Rangées):
        Dic = {}
        for c in range(1, Colonnes):
            Val = ExcelSheet.cell(row=r, column=c).value
            if type(Val) == str:
                if '=HYPERLINK(' in Val:
                    Val = Val.replace('=HYPERLINK(', '').replace(')', '').split(', ')[0].replace('"', '')
            Dic[ExcelSheet.cell(row=1, column=c).value] = Val
        Retour.append(Dic)

    return Retour

def CSV(tr):
    Retour = []

    CSVFile = open(tr, 'r').read()
    CSVTitles = CSVFile.split('\n')[:1][0].split(',')
    CSVLine = CSVFile.split('\n')[1:]
    for r in CSVLine:
        Dic = {}
        c = r.split(',')
        for i, e in enumerate(c):
            Dic[CSVTitles[i]] = e.replace('<breakline>', '\n').replace('<coma>', ',')
        Retour.append(Dic)

    return Retour[:-1]

def Json(tr):
    return json.loads(open(tr, 'r').read())

def verter(entry) -> list:
    '''- entry: can be path of the file to convert to dict, also a dict to convert to the comparaison module format'''
    try:
        ext = entry.split('.')[-1]
        if re.match(r'xlsx', ext):
            return excel(entry)
        elif re.match(r'csv', ext):
            return CSV(entry)
        elif re.match(r'json', ext):
            return Json(entry)
    
    except FileNotFoundError:
        outExcept(e=f'Conversion module didn\'t find the file {entry}!\n    Be sure of your path.')


class lutio:
    def __init__(self, f1, f2, arg) -> None:
        self.f1 = f1
        self.f2 = f2
        self.rg = arg

    def not_twice(l) -> list:
        r = []
        for i in l:
            if i not in r:
                r.append(i)
        return r

    def n(self) -> list:
        '''comparing the id of the account to avoid not equal dict between default and all-infos datas,
        and to avoid not same account between two list if an username changed its biography or something else'''
        if self.rg == 'common_usernames':
            common_usernames = []
            u2ids = [U['id'] for U in self.f2]
            for u1 in self.f1:
                if u1['id'] in u2ids:
                    common_usernames.append(u1)
            
            return lutio.not_twice(common_usernames)
        
        elif self.rg == 'not_common_usernames':
            not_common_usernames = []
            u2ids = [U['id'] for U in self.f2]
            for u1 in self.f1:
                if not u1['id'] in u2ids:
                    not_common_usernames.append(u1)
            
            return lutio.not_twice(not_common_usernames)


import sys
import os
import argparse

from sterra import _exp
from sterra import _pri
from sterra import _con
from sterra import _ins

from stringcolor import *

def main():
    DP = os.path.dirname(__file__)+'/export/'
    if not os.path.exists(DP):
        os.mkdir(DP)
    if not os.path.exists(DP+'parts/'):
        os.mkdir(DP+'parts/')

    def d():
        print(underline(bold('Please choose a module from the followings:')))
        e = cs('-> ', 'Red2')
        print(f'''    {e}export   (Export list of followers / following / mutuals of an instagram account)
    {e}analyse  (Obtain probabilities for each accounts of being personnal accounts)
    {e}compare  (Shows accounts present in first list and not in the second)
    {e}history  (Check the historic containing all the path of the file you exported, in case you forgot it)
    {e}convert  (Convert a file already exported to another format)''')
        exit()
    try:
        b = sys.argv[1].lower()
    except IndexError:
        d()

    try:
        print(_pri.logo(b))
    except KeyError:
        pass

    if b == 'export':
        try:
            if sys.argv[2].lower() == 'file':
                export = argparse.ArgumentParser()
                handle1 = export.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: export
                handle2 = export.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: file

                export.add_argument('-f', '--format', choices=['excel', 'csv', 'json'], help='format of the export, by default "excel"')
                export.add_argument('--parts', nargs=(1 or 2), help='path(s) to the complete file(s)')
                export.add_argument('-t', '--target', choices=["followers", "following", "both", "mutuals"], help='what do you want to export ("followers", "following", "both" or "mutuals")', required=True)
                export.add_argument('-p', '--path', metavar='P', help='directory path where export the files (by default in your module path)')
                exargs = export.parse_args()

                exargs.format = "excel" if exargs.format is None else exargs.format
                exargs.path = DP if exargs.path is None else exargs.path
                logo = _pri.Excel() if exargs.format == 'excel' else _pri.CSV() if exargs.format == 'csv' else _pri.Json()
                if not os.path.exists(exargs.path):
                    os.mkdir(exargs.path)

                def make_the_job(path):
                    print(f'{_pri.i()}Conversion ...')
                    j = [_ins.traction.keep_essential_data(False, E) for E in _con.verter(path)]
                    i = _con.getnameinfos(path)
                    return j, i

                if exargs.target == 'followers' or exargs.target == 'following':
                    j, infos = make_the_job(exargs.parts[0])
                    _exp.ort(j, exargs.path, exargs.format, exargs.target, infos['username']).d()
                else:
                    if len(exargs.parts) != 2:
                        _pri.outExcept(e='Argument --parts require two part lists (following and followers)')                
                    
                    l1, i1 = make_the_job(exargs.parts[0])
                    l2, i2 = make_the_job(exargs.parts[1])
                    if exargs.target == 'mutuals':
                        mutuals = _ins.makemutuals(l1, l2)
                        _exp.ort(mutuals, exargs.path, exargs.format, exargs.target).d()
                    
                    else:
                        followers = l1 if i1['type'] == 'followers' else l2
                        following = l2 if i2['type'] == 'following' else l1

                        _exp.ort(followers, exargs.path, exargs.format, exargs.target)
                        _exp.ort(following, exargs.path, exargs.format, exargs.target)
            else:
                raise IndexError
        
        except IndexError:
            export = argparse.ArgumentParser(add_help=False, usage='sterra (-u/--username U | -id ID) -t/--target {followers,following,both,mutuals} (-lcrd/--login-credentials U P | -ssid/--login-session-id S) [--all-infos] -f/--format {excel,csv,json} [-l/--local | -p/--path P] [-d/--delay DELAY | -e/--express] [-h/--help] [--part "PATH_TO_PART"]\n\nWiki: https://github.com/novitae/STERRADEV/blob/main/WIKI.md#export')
            handle = export.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: export

            # TARGET & TARGETLIST
            required = export.add_argument_group(title='required arguments')
            rexclued = required.add_mutually_exclusive_group(required=True)
            rexclued.add_argument('-u', '--username', metavar='U', help='the instagram username of the aimed account', type=str)
            rexclued.add_argument('-id', metavar='ID', help='the instagram id of the aimed account', type=int)
            required.add_argument('-t', '--target', choices=["followers", "following", "both", "mutuals"], help='what do you want to export ("followers", "following", "both" or "mutuals")', required=True)
            
            # LOGIN
            login = export.add_argument_group(title='login arguments')
            lognn = login.add_mutually_exclusive_group(required=True)
            lognn.add_argument('-lcrd', '--login-credentials', metavar=('U', 'P'), nargs=2, help='login by credentials: USERNAME PASSWORD (be sure to keep a space between them)')
            lognn.add_argument('-ssid', '--login-session-id', metavar='S', help='login by SessionID')
            
            # EXPORT
            exports = export.add_argument_group(title='export arguments')
            exports.add_argument('--all-infos', action='store_true', help='writes down the account extra informations that the program originaly ignores')
            exports.add_argument('-f', '--format', choices=['excel', 'csv', 'json'], help='format of the export, by default "excel"')
            exports.add_argument('-p', '--path', metavar='P', help='directory path where export the files (by default in your module path)')

            # SPEED
            spdnm = export.add_argument_group(title='speed arguments')
            speed = spdnm.add_mutually_exclusive_group()
            speed.add_argument('-d', '--delay', type=int, help='delay between requests to get accounts informations')
            speed.add_argument('-e', '--express', action='store_true', help='sends ultra fast requests to get the table faster (deactivated if more than 109 total usernames to avoid blocking)')

            # OPTIONAL
            optional = export.add_argument_group(title='optional arguments')
            optional.add_argument('-h', '--help', action='help', default=argparse.SUPPRESS, help='show this help message and exit')
            optional.add_argument('--part', type=str, help='path of the part')

            exargs = export.parse_args()

            # SET DEFAULTS
            exargs.format = "excel" if exargs.format is None else exargs.format
            exargs.express = False if exargs.express is None else exargs.express
            exargs.path = DP if exargs.path is None else exargs.path
            if not os.path.exists(exargs.path):
                print(exargs.path)
                os.mkdir(exargs.path)

            # EXTRACTION PROCESS
            _ins.tagram(
                tu=(exargs.username, 'username') if exargs.id is None else (exargs.id, 'id'),
                cd=exargs.login_credentials if exargs.login_session_id is None else exargs.login_session_id,
                tl=exargs.target,
                pt=exargs.path,
                ex=exargs.express,
                ai=exargs.all_infos,
                fm=exargs.format,
                rm=exargs.part
            )
        
    elif b == 'compare':
        compare = argparse.ArgumentParser(usage='sterra -f1 PATH_TO_FIRST_FILE -f2 PATH_TO_SECOND_FILE [--no-print] [--url] [--common-usernames] [--not-common-usernames] [-e] [-f {excel,csv,json}] [-p EXPORT_PATH]\n\nWiki: https://github.com/novitae/STERRADEV/blob/main/WIKI.md#compare')
        handle = compare.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: compare

        compare.add_argument('-f1', '--file-1', help='path to the first file to compare to the second file', required=True)
        compare.add_argument('-f2', '--file-2', help='path to the second file to compare to the first file', required=True)
        compare.add_argument('-n', '--no-print', action='store_true', help='doesn\'t print results')
        compare.add_argument('-u', '--url', action='store_true', help='instead of printing usernames, printing urls to the accounts')

        want = compare.add_argument_group(title='data to get')
        want.add_argument('--common-usernames', action='store_true', help='shows the commons usernames (compared with instagram id) in lists')
        want.add_argument('--not-common-usernames', action='store_true', help='shows the not commons usernames in lists')

        expt = compare.add_argument_group(title='export data')
        expt.add_argument('-e', '--export', action='store_true', help='exports the data returned')
        expt.add_argument('-f', '--format', choices=['excel', 'csv', 'json'], help='format of the exported file')
        expt.add_argument('-p', '--path', help='path to the exported file')
        cpargs = compare.parse_args()

        cpargs.format = cpargs.format if cpargs.format is not None else 'excel'
        cpargs.path = cpargs.path if cpargs.path is not None else DP

        f1 = _con.verter(cpargs.file_1)
        f2 = _con.verter(cpargs.file_2)

        lag = ['common_usernames', 'not_common_usernames']
        arglist = []
        for i, tf in enumerate(list(vars(cpargs).keys())):
            if tf in lag and list(vars(cpargs).values())[i] is True:
                arglist.append(tf)

        from sterra import _evo
        for arg in arglist:
            data = _evo.lutio(f1, f2, arg).n()
            if not cpargs.no_print:
                print(_pri.compare()+underline(bold(arg.replace('_', ' ').capitalize()+' :')))
                if data == []:
                    print(bold(cs(' >  No '+arg.replace('_', ' ')+' were found.', 'Red4')))
                else:
                    for dc in data:
                        dt = cs('https://www.instagram.com/', 'LightGrey13')+bold(dc['username'])+cs('/', 'LightGrey13') if cpargs.url else dc['username']
                        print(cs(' >  ', 'DeepSkyBlue7').bold()+dt)
        
            if cpargs.export:
                dt = {
                    'not_common_usernames': f'usernames in {cpargs.file_1} but not in {cpargs.file_2}',
                    'common_usernames': f'usernames in {cpargs.file_1} and also in {cpargs.file_2}'
                }
                _exp.ort(
                    ld=data,
                    tp='comparaison',
                    fm=cpargs.format,
                    tu=str(_con.getnameinfos(cpargs.file_1)['username'])+' & '+str(_con.getnameinfos(cpargs.file_2)['username']),
                    pt=cpargs.path,
                    dt=dt[arg]
                    ).d()

            # shows the accounts present in l1 and not in l2

    elif b == 'analyse':
        analyse = argparse.ArgumentParser(usage='-p/--path PATH [-a/--analysis-type {personnal}] [-d/--descending] [-e/--export] [-f/--format] [-i/--ignore-over] [-n/--no-print] [--pctg] [-s/--size] [-u/--url]\n\nWiki: https://github.com/novitae/STERRADEV/blob/main/WIKI.md#analyse')
        handle = analyse.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: analyse
        
        requ = analyse.add_argument_group(title='required arguments')
        requ.add_argument('-p', '--path', help='path to the file to analyse', required=True)
        
        analyse.add_argument('-a', '--analysis-type', choices=['personnal'], help='type of probabilities asked') # BIENTOT "INTERESTS"
        analyse.add_argument('-d', '--descending', action='store_true', help='instead of printing by highest probability, printing by lowest probability')
        analyse.add_argument('-e', '--export', action='store_true', help='export results')
        analyse.add_argument('-f', '--format', choices=['excel', 'csv', 'json'], help='file export format')
        analyse.add_argument('-i', '--ignore-over', help='quantity of followers over wich we consider the account as impossible to be the close circle of the target (default: 10000)', type=int)
        analyse.add_argument('-n', '--no-print', action='store_true', help='doesn\'t print results')
        analyse.add_argument('--pctg', help='percentage under wich we won\'t print results (between 0 and 98)', type=int)
        analyse.add_argument('-s', '--size', help='size of the most probable username list (will be by default the size of the followers/mutuals/following list filled in)', type=int)
        analyse.add_argument('-u', '--url', action='store_true', help='instead of printing the username of the account, prints the url')
        anargs = analyse.parse_args()

        anargs.analysis_type = anargs.analysis_type if anargs.analysis_type is not None else 'personnal'
        anargs.ignore_over = anargs.ignore_over if anargs.ignore_over is not None else 10000
        anargs.pctg = anargs.pctg if anargs.pctg is not None else 0
        anargs.format = anargs.format if anargs.format is not None else 'excel'

        ld = _con.verter(anargs.path)
        from sterra import _pro
        probas = _pro.babilities(ld=ld, pa=anargs.analysis_type, io=anargs.ignore_over, dd=anargs.descending, pt=anargs.pctg, sz=len(ld) if anargs.size is None else anargs.size).d()
        
        if not anargs.no_print:
            def setcolor(i):
                if i > 85: return 'Green3'
                elif i > 80: return 'Green2'
                elif i > 75: return 'Chartreuse2'
                elif i > 70: return 'GreenYellow'
                elif i > 65: return 'Yellow4'
                elif i > 60: return 'Yellow'
                elif i > 55: return 'LightGoldenrod2'
                elif i > 50: return 'Orange'
                elif i > 45: return 'DarkOrange'
                elif i > 40: return 'OrangeRed'
                elif i > 35: return 'Red2'
                elif i > 30: return 'Red3'
                elif i > 25: return 'Red4'
                elif i > 20: return 'DarkRed2'
                else: return 'DarkRed'

            print(_pri.i()+cs.bold(' Probabilities')+' for '+bold(anargs.path)+' accounts of '+bold('being '+anargs.analysis_type+' account')+f' (showing {len(probas)}/{len(ld)}):')
            print(cs.bold(' >>> '+cs('|', 'DarkGreen')+bold('%%%').cs('White', 'DarkGreen')+cs('|', 'DarkGreen')+' = '+cs.bold('Username')))
            if probas == []:
                nn = cs('->', setcolor(30))
                print(f'      !!! {nn} '+bold('empty list'))
            else:
                for e in probas:
                    lineco = setcolor(int(e['%']))
                    print('      '+bold(e['%'])+'%'+cs(' -> ', lineco)+cs((e['username'] if not anargs.url else cs('https://www.instagram.com/', 'LightGrey9')+bold(e['username'])+cs('/', 'LightGrey9')), 'White').underline())

        if anargs.export:
            fo = _con.getnameinfos(anargs.path)
            t = fo['type']
            _exp.ort(ld=probas, tp=f'{t}_{anargs.analysis_type}_probabilities', fm=anargs.format, tu=fo['username'], pt=fo['path']).d()

    elif b == 'history':
        history = argparse.ArgumentParser(usage='sterra history (--all | --match | --clear)\n\nWiki: https://github.com/novitae/STERRADEV/blob/main/WIKI.md#history')
        handle = history.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: history
        
        choice = history.add_mutually_exclusive_group(required=True)
        choice.add_argument('-a', '--all', action='store_true', help='will print chronologically all the exports path saved in the history')
        choic2 = choice.add_mutually_exclusive_group()
        choic2.add_argument('-m', '--match', help='will print all the exports containing the string attached to this arg (can even be .xlsx to print by format)')
        choic2.add_argument('-c', '--clear', action='store_true', help='clear the historic (you can\'t go back after doing it)')
        hiargs = history.parse_args()

        from sterra import _his
        if hiargs.all:
            _his.tory(ll=True)
        elif hiargs.match is not None:
            _his.tory(nm=hiargs.match)
        elif hiargs.clear:
            _his.tory(cl=True)

    elif b == 'convert':
        convert = argparse.ArgumentParser(usage='sterra convert -p/--path PATH -f/--format {excel, csv, json}\n\nWiki: https://github.com/novitae/STERRADEV/blob/main/WIKI.md#convert')
        handle = convert.add_argument(' ', help=argparse.SUPPRESS) # Avoid => parsers.py: error: unrecognized arguments: convert

        convert.add_argument('-p', '--path', help='path of the file to convert', required=True)
        convert.add_argument('-f', '--format', choices=['excel', 'csv', 'json'], help='format of the convert', required=True)
        coargs = convert.parse_args()

        fo = _con.getnameinfos(coargs.path)
        bold(_exp.ort(ld=_con.verter(coargs.path), tp=fo['type'], fm=coargs.format, tu=fo['username'], pt=fo['path']).d())

    else:
        d()

    print('[ € ] ~['+cs(' BTC ', 'MediumPurple6').bold()+cs('bc1qjdw2hsspdlw7j9j9qn24gnujnk5thdmt6h2kjh', 'DarkSeaGreen9').underline()+cs(' thx <3 ', 'MediumPurple6').bold()+']~ ~ ~ ~')

if __name__ == '__main__':
    main()


from setuptools import setup, find_packages

setup(
    name = 'sterratest',
    version = '3.0.15',
    description = 'OSINT tool to work with follower/following list of instagram users.',
    long_description = 'OSINT tool to work with follower/following list of instagram users, see README of https://github.com/novitae/sterraxcyl',
    author = 'novitae',
    url = 'https://github.com/novitae/sterraxcyl',
    licence = 'GNU General Public License v3 (GPLv3)',
    classifiers = [
        'Programming Language :: Python :: 3.9',
    ],
    packages = find_packages(),
    install_requires = ['aiohttp', 'argparse', 'datetime', 'openpyxl', 'requests', 'string-color', 'tqdm'],
    entry_points = {'console_scripts': ['sterra = sterra.core:main']}
)